unit ReactionTime_unit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ComCtrls, StdCtrls,
  ExtCtrls, Grids, Buttons, SLHColorPicker, IniFiles, LCLType, Clipbrd, Math, BGRABitmap, BGRABitmapTypes, epiktimer, Menus,

  //  Displays file version info for
  //- Windows PE executables
  //- Linux ELF executables (compiled by Lazarus)
  //- macOS MACH-O executables (compiled by Lazarus)
  //  Runs on Windows, Linux, macOS
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  // FPC 3.0 fileinfo reads exe resources as long as you register the appropriate units
  fileinfo,
  winpeimagereader, //need this for reading exe info
  elfreader, //needed for reading ELF executables
  machoreader; //needed for reading MACH-O executables



type

  { TMainForm }

  TMainForm = class(TForm)
    AutoViewDataTableCheckBox: TCheckBox;
    AboutMemo: TMemo;
    MainMenu: TMainMenu;
    ShowVirtualMouseButtonsCheckBox: TCheckBox;
    NumberOfMeasurementsEdit: TEdit;
    BackgroundBlackPanel: TPanel;
    StimulusColorLabel: TLabel;
    ShowInstructionsCheckBox: TCheckBox;
    ShowFixationPointCheckBox: TCheckBox;
    CopyDataToClipboardButton: TButton;
    ClearDataTableButton: TButton;
    CircleImage: TImage;
    DelayForStimulusShowupStartEdit: TEdit;
    DelayForStimulusShowupEndEdit: TEdit;
    FixationPointImage: TImage;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    RightButtonRoundImage: TImage;
    LeftButtonRoundImage: TImage;
    InstructionsLabel: TLabel;
    PageControl1: TPageControl;
    DataStringGrid: TStringGrid;
    StimulurSLHColorPicker: TSLHColorPicker;
    TabItemExperiment: TTabSheet;
    TabItemData: TTabSheet;
    TabItemOptions: TTabSheet;
    TabItemAbout: TTabSheet;
    Timer1: TTimer;
    procedure BackgroundBlackPanelMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure BackgroundBlackPanelMouseUp(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure BackgroundImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BackgroundImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CircleImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CircleImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ClearDataTableButtonClick(Sender: TObject);
    procedure CopyDataToClipboardButtonClick(Sender: TObject);
    procedure DelayForStimulusShowupEndEditEnter(Sender: TObject);
    procedure DelayForStimulusShowupEndEditExit(Sender: TObject);
    procedure DelayForStimulusShowupEndEditKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure DelayForStimulusShowupStartEditEnter(Sender: TObject);
    procedure DelayForStimulusShowupStartEditExit(Sender: TObject);
    procedure DelayForStimulusShowupStartEditKeyDown(Sender: TObject;
      var Key: Word; Shift: TShiftState);
    procedure FixationPointImageMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FixationPointImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure InstructionsLabelMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure InstructionsLabelMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LeftButtonRoundImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LeftButtonRoundImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure NumberOfMeasurementsEditEnter(Sender: TObject);
    procedure NumberOfMeasurementsEditExit(Sender: TObject);
    procedure NumberOfMeasurementsEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RightButtonRoundImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RightButtonRoundImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ShowFixationPointCheckBoxChange(Sender: TObject);
    procedure ShowInstructionsCheckBoxChange(Sender: TObject);
    procedure ShowVirtualMouseButtonsCheckBoxChange(Sender: TObject);
    procedure StimulurSLHColorPickerChange(Sender: TObject);
    procedure TabItemOptionsShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private

  public
    procedure GetScreenSize();
    function CheckSettings(): boolean;
  end;

var
  MainForm: TMainForm;


first_button_clicked, second_button_clicked, stimulus_shown: boolean;
delay_for_stimulus_showup_start, delay_for_stimulus_showup_start_saved, delay_for_stimulus_showup_end, delay_for_stimulus_showup_end_saved: Integer;
delay_for_stimulus_showup: Integer;
delay_for_stimulus_showup_in_seconds: Double;
stop_watch_visual_elapsed_in_ms, stop_watch_2nd_button_elapsed_in_ms: Double;


step_data_point: Integer;
number_of_measurements, number_of_measurements_saved: Integer;

number_of_measurements_default, delay_for_stimulus_showup_start_default, delay_for_stimulus_showup_end_default: Integer;


CircleBitmap: TBGRABitmap;
FixationPointBitmap: TBGRABitmap;
VirtualMouseButtonRoundBitmap: TBGRABitmap;
IncreasedVirtualMouseButtonRoundBitmap: TBGRABitmap;


device_type: Integer;
screen_scale: Double;
main_form_width, main_form_height: Integer;
fix_main_form_height, fix_main_form_width: Double;
screen_orientation_int: Integer;
old_screen_orientation_int: Integer;

ProgramIniFile: TIniFile;
ini_filename, ini_filepath, exe_filepath, application_path, test_path: TFileName;
ini_int_result: Integer;
ini_bool_result: boolean;
ini_string_result: String;

button_increase_ratio: Double;
left_button_rectangle_center_x, right_button_rectangle_center_x: Integer;
left_button_rectangle_center_y, right_button_rectangle_center_y: Integer;

left_button_rectangle_position_x, right_button_rectangle_position_x: Integer;
left_button_rectangle_position_y, right_button_rectangle_position_y: Integer;

show_virtual_mouse_buttons: boolean;
button_width, button_height: Integer;
increased_button_width, increased_button_height: Integer;


stop_watch_delay_stimulus, stop_watch_visual, stop_watch_2nd_button: TEpikTimer; //TStopwatch; //System::Diagnostics::TStopwatch;

FileVerInfo: TFileVersionInfo;


implementation

{$R *.lfm}

{ TMainForm }



procedure TMainForm.GetScreenSize();
begin

  main_form_height := MainForm.Height;
  main_form_width := MainForm.Width;

end;


function TMainForm.CheckSettings(): boolean;
begin

  if(StrToInt(DelayForStimulusShowupStartEdit.Text) >= StrToInt(DelayForStimulusShowupEndEdit.Text)) then
  begin
    ShowMessage('''from'' time must be less than ''to'' time');
    result := false;
  end
  else
  begin


  number_of_measurements := StrToIntDef(NumberOfMeasurementsEdit.Text, number_of_measurements_default);
  if((number_of_measurements <= 0) and (number_of_measurements > 1000)) then
  begin
    result := false;
    Exit;
  end;


  delay_for_stimulus_showup_start := StrToIntDef(DelayForStimulusShowupStartEdit.Text, delay_for_stimulus_showup_start_default);
  if((delay_for_stimulus_showup_start < 0) or (delay_for_stimulus_showup_start > 100000)) then
  begin
    result := false;
    Exit;
  end;

  delay_for_stimulus_showup_end := StrToIntDef(DelayForStimulusShowupEndEdit.Text, delay_for_stimulus_showup_end_default);
  if((delay_for_stimulus_showup_end < 0) or (delay_for_stimulus_showup_end > 1000000)) then
  begin
    result := false;
    Exit;
  end;

  result := true;
  end;



end;



procedure TMainForm.FormCreate(Sender: TObject);
{$IFDEF DARWIN}
var
     AppMenu     : TMenuItem;
{$ENDIF}
begin


  {$IFDEF DARWIN}
  AppMenu := TMenuItem.Create(Self); // Application menu
  AppMenu.Caption := #$EF#$A3#$BF;   // Unicode Apple logo char
  MainMenu.Items.Insert(0, AppMenu);
  {$ENDIF}



  number_of_measurements_default := 30;
  delay_for_stimulus_showup_start_default := 1000;
  delay_for_stimulus_showup_end_default := 10000;


  CircleImage.Visible := false;
  InstructionsLabel.Font.Size := 24;

  button_increase_ratio := 1.3;

  button_width := 66;
  button_height := 120;

  increased_button_width := Ceil(button_width * button_increase_ratio);
  increased_button_height := Floor(button_height * button_increase_ratio);



  PageControl1.ActivePage := TabItemExperiment;


// https://wiki.lazarus.freepascal.org/EpikTimer
// Nanosecond resolution is supported on Intel Pentium versions with a Timestamp Counter.
// Microsecond system clock is the default timebase.

  stop_watch_delay_stimulus := TEpikTimer.Create(Application); // TStopwatch.Create();
  stop_watch_delay_stimulus.Clear();

  stop_watch_visual :=  TEpikTimer.Create(Application); // TStopwatch.Create();
  stop_watch_visual.Clear();

  stop_watch_2nd_button :=  TEpikTimer.Create(Application); // TStopwatch.Create();
  stop_watch_2nd_button.Clear();


  GetScreenSize();





  first_button_clicked := false;
  second_button_clicked := false;
  stimulus_shown := false;



  //=======



  {$IF Defined (DARWIN)} // macOS
  ini_filepath := GetUserDir();
  ini_filepath := ini_filepath + 'Library/Application Support/Cognaxon/ReactionTimeMeasurement/';

  if(ForceDirectories(ini_filepath)) then
  begin
  ini_filename := ini_filepath + 'ReactionTimeMeasurement.ini';
  end;
  {$ELSEIF Defined (WINDOWS)}

  // https://www.freepascal.org/docs-html/rtl/sysutils/getappconfigdir.html
  // C:\Users\mantas\AppData\Roaming\Cognaxon\ReactionTimeMeasurement
  // C:\Users\mantas\AppData\Local\ReactionTimeMeasurement\Cognaxon\ReactionTimeMeasurement

  //ini_filepath := GetAppConfigDir(true);
  ini_filepath := GetAppConfigDir(false);
  ini_filepath := ExtractFilePath(ExcludeTrailingPathDelimiter(ini_filepath)); //parentDirPath
  ini_filepath := ini_filepath + 'Cognaxon\ReactionTimeMeasurement\';

  if(ForceDirectories(ini_filepath)) then
  begin
  ini_filename := ini_filepath + 'ReactionTimeMeasurement.ini';
  end;

  {$ELSE}
   ini_filename := ChangeFileExt( ParamStr(0), '.ini' );
  {$ENDIF}




//=======
  try
  ProgramIniFile := TIniFile.Create(ini_filename);

  number_of_measurements := ProgramIniFile.ReadInteger( 'ReactionTime', 'number_of_measurements', number_of_measurements_default);

  if((number_of_measurements < 0) or (number_of_measurements > 10000)) then // protection from crash
  begin
  number_of_measurements := number_of_measurements_default;
  end;


  delay_for_stimulus_showup_start := ProgramIniFile.ReadInteger( 'ReactionTime', 'delay_for_stimulus_showup_start', delay_for_stimulus_showup_start_default);

  if((delay_for_stimulus_showup_start < 0) or (delay_for_stimulus_showup_start > 100000)) then // protection from crash
  begin
  delay_for_stimulus_showup_start := delay_for_stimulus_showup_start_default;
  end;


  delay_for_stimulus_showup_end := ProgramIniFile.ReadInteger( 'ReactionTime', 'delay_for_stimulus_showup_end', delay_for_stimulus_showup_end_default);

  if((delay_for_stimulus_showup_end < 0) or (delay_for_stimulus_showup_end > 1000000)) then // protection from crash
  begin
  delay_for_stimulus_showup_end := delay_for_stimulus_showup_end_default;
  end;


  ShowInstructionsCheckBox.Checked := ProgramIniFile.ReadBool( 'ReactionTime', 'show_instructions', true);
  InstructionsLabel.Visible := ShowInstructionsCheckBox.Checked;

  ShowFixationPointCheckBox.Checked := ProgramIniFile.ReadBool( 'ReactionTime', 'show_fixation_point', true);
  FixationPointImage.Visible := ShowFixationPointCheckBox.Checked;

  AutoViewDataTableCheckBox.Checked := ProgramIniFile.ReadBool( 'ReactionTime', 'auto_view_data_table', true);

  ShowVirtualMouseButtonsCheckBox.Checked := ProgramIniFile.ReadBool( 'ReactionTime', 'show_virtual_mouse_buttons', false);

  RightButtonRoundImage.Visible := show_virtual_mouse_buttons;
  LeftButtonRoundImage.Visible := show_virtual_mouse_buttons;


  StimulurSLHColorPicker.OnChange := nil;
  StimulurSLHColorPicker.SelectedColor := StringToColor(ProgramIniFile.ReadString( 'ReactionTime', 'stimulus_color', '$00FFFFFF'));
  StimulurSLHColorPicker.OnChange := @StimulurSLHColorPickerChange;


finally
 ProgramIniFile.Free;
end;

//=======




// https://www.pascalgamedevelopment.com/archive/index.php/t-8088.html
// I've added CanvasBGRA property to TBGRABitmap to allow drawing with functions that work the same as the standard Canvas but with antialiasing and alphablending.
// The AntialiasingMode is always taken into account and there is an Opacity property added in Pen, Brush and Font.

  CircleBitmap := TBGRABitmap.Create(200, 200, clNone);
  CircleBitmap.CanvasBGRA.Pen.Color := StimulurSLHColorPicker.SelectedColor;
  CircleBitmap.CanvasBGRA.Pen.Style := psSolid;
  CircleBitmap.CanvasBGRA.Pen.Width := 0;
  CircleBitmap.CanvasBGRA.Brush.Color := StimulurSLHColorPicker.SelectedColor;
  CircleBitmap.CanvasBGRA.Brush.Style := bsSolid;
  CircleBitmap.CanvasBGRA.Ellipse(0, 0, CircleBitmap.Width, CircleBitmap.Height);


  CircleImage.Width := CircleBitmap.Width;
  CircleImage.Height := CircleBitmap.Height;
  CircleImage.Picture.Bitmap.Assign(CircleBitmap);


  FixationPointBitmap := TBGRABitmap.Create(10, 10, clNone);
  FixationPointBitmap.CanvasBGRA.Pen.Color := clWhite;
  FixationPointBitmap.CanvasBGRA.Pen.Style := psSolid;
  FixationPointBitmap.CanvasBGRA.Pen.Width := 0;
  FixationPointBitmap.CanvasBGRA.Brush.Color := clWhite;
  FixationPointBitmap.CanvasBGRA.Brush.Style := bsSolid;
  FixationPointBitmap.CanvasBGRA.Ellipse(0, 0, FixationPointBitmap.Width, FixationPointBitmap.Height);

  FixationPointImage.Width := FixationPointBitmap.Width;
  FixationPointImage.Height := FixationPointBitmap.Height;
  FixationPointImage.Picture.Bitmap.Assign(FixationPointBitmap);
//  FixationPointImage.BringToFront();


VirtualMouseButtonRoundBitmap := TBGRABitmap.Create(button_width, button_height, clNone);
VirtualMouseButtonRoundBitmap.CanvasBGRA.Pen.Color := clWhite;
VirtualMouseButtonRoundBitmap.CanvasBGRA.Pen.Style := psSolid;
VirtualMouseButtonRoundBitmap.CanvasBGRA.Pen.Width := 0;
VirtualMouseButtonRoundBitmap.CanvasBGRA.Brush.Color := clWhite;
VirtualMouseButtonRoundBitmap.CanvasBGRA.Brush.Style := bsSolid;
VirtualMouseButtonRoundBitmap.CanvasBGRA.Ellipse(0, 0, button_width, button_width);
VirtualMouseButtonRoundBitmap.CanvasBGRA.Ellipse(0, button_height-button_width, button_width, button_height);
VirtualMouseButtonRoundBitmap.CanvasBGRA.Rectangle(0, button_width div 2, button_width, button_height - button_width div 2);

IncreasedVirtualMouseButtonRoundBitmap := TBGRABitmap.Create(increased_button_width, increased_button_height, clNone);
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Pen.Color := clWhite;
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Pen.Style := psSolid;
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Pen.Width := 0;
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Brush.Color := clWhite;
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Brush.Style := bsSolid;
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Ellipse(0, 0, increased_button_width, increased_button_width);
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Ellipse(0, increased_button_height-increased_button_width, increased_button_width, increased_button_height);
IncreasedVirtualMouseButtonRoundBitmap.CanvasBGRA.Rectangle(0, increased_button_width div 2, increased_button_width, increased_button_height - increased_button_width div 2);


RightButtonRoundImage.Width := button_width;
RightButtonRoundImage.Height := button_height;
RightButtonRoundImage.Picture.Bitmap.Assign(VirtualMouseButtonRoundBitmap);

LeftButtonRoundImage.Width := button_width;
LeftButtonRoundImage.Height := button_height;
LeftButtonRoundImage.Picture.Bitmap.Assign(VirtualMouseButtonRoundBitmap);



  DataStringGrid.ColCount := 3;
  DataStringGrid.RowCount := number_of_measurements + 1;


  DataStringGrid.ColWidths[0] := 60;
  DataStringGrid.ColWidths[1] := 80;
  DataStringGrid.ColWidths[2] := 80;

  DataStringGrid.Cells[0, 0] := 'Number';
  DataStringGrid.Cells[1, 0] := 'Stimulus';
  DataStringGrid.Cells[2, 0] := '2nd button';



  step_data_point := 1;


  //ClearDataTableButton.BringToFront();
  //CopyDataToClipboardButton.BringToFront();
  //DataStringGrid.BringToFront();



  AboutMemo.WordWrap := true;
  AboutMemo.Lines.Clear();

  FileVerInfo := TFileVersionInfo.Create(nil);

  try
  FileVerInfo.ReadFileInfo;
  //AboutMemo.Lines.Add('Reaction Time Measurement v. ' + FileVerInfo.VersionStrings.Values['FileVersion']);
  AboutMemo.Lines.Add('Reaction Time Measurement v. ' + FileVerInfo.VersionStrings.Values['ProductVersion']);
  finally
    FileVerInfo.Free;
  end;

  AboutMemo.Lines.Add('');
  AboutMemo.Lines.Add('A tool for psychophysics laboratory experiments.');
  AboutMemo.Lines.Add('Measures reaction time between:');
  AboutMemo.Lines.Add(' 1) appearance of visual stimulus and the release of the first button;');
  AboutMemo.Lines.Add(' 2) the release of the first button and pressing of the second button.');
  AboutMemo.Lines.Add('');
  AboutMemo.Lines.Add('');
  AboutMemo.Lines.Add('"Reaction Time Measurement" is free open-source cross-platform software made by Cognaxon');
  AboutMemo.Lines.Add('http://cognaxon.com');
  AboutMemo.Lines.Add('');
  AboutMemo.Lines.Add('Last updated: 2021-06-23');
  AboutMemo.Lines.Add('');
  AboutMemo.Lines.Add('Please check out official "Neurocluster Brain Model" site which is available at the address:');
  AboutMemo.Lines.Add('http://neuroclusterbrain.com');
  AboutMemo.Lines.Add('');
  AboutMemo.Lines.Add('INI file location:');
  AboutMemo.Lines.Add(ini_filename);






  AboutMemo.Visible := true;

  AboutMemo.BringToFront();




end;

procedure TMainForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin


  if(Timer1.Enabled = true) then
  begin

  if(Button = TMouseButton.mbLeft) then
  begin

      Randomize();
      delay_for_stimulus_showup := delay_for_stimulus_showup_start + Random(delay_for_stimulus_showup_end - delay_for_stimulus_showup_start);
      delay_for_stimulus_showup_in_seconds := delay_for_stimulus_showup / 1000.0;


      {$IFDEF ANDROID}
      InstructionsLabel.Caption := 'Keep left button pressed ' + LineEnding + 'until circle shows up. ' + LineEnding + 'Then release left button and press right ' + LineEnding + 'button as quickly as you can.';
      {$ELSE}
      InstructionsLabel.Caption := 'Keep left mouse button pressed ' + LineEnding + 'until circle shows up. ' + LineEnding + 'Then release left mouse button and press right ' + LineEnding + 'mouse button as quickly as you can.';
      {$ENDIF}


      InstructionsLabel.Repaint();
      Application.ProcessMessages();

      stop_watch_2nd_button.Stop();
      stop_watch_2nd_button.Clear();

      stop_watch_delay_stimulus.Stop();
      stop_watch_delay_stimulus.Clear();
      stop_watch_delay_stimulus.Start();

      first_button_clicked := true;
      stimulus_shown := false;

      second_button_clicked := false;

  end//  if(Button = TMouseButton.mbLeft)
  else if(Button = TMouseButton.mbRight) then
  begin


    if(second_button_clicked = true) then
    begin
      Exit;
    end;

     stop_watch_2nd_button.Stop();
     stop_watch_2nd_button_elapsed_in_ms := RoundTo(stop_watch_2nd_button.Elapsed * 1000.0, -3);

     DataStringGrid.Cells[0, step_data_point] := IntToStr(step_data_point);
     DataStringGrid.Cells[1, step_data_point] := FloatToStr(stop_watch_visual_elapsed_in_ms);
     DataStringGrid.Cells[2, step_data_point] := FloatToStr(stop_watch_2nd_button_elapsed_in_ms);
     Inc(step_data_point);


       if(step_data_point > number_of_measurements) then
       begin
         CircleImage.Visible := false;
  	 InstructionsLabel.Caption := 'Reaction time measurements completed. ' + LineEnding + 'Use data table to view measured data.';
  	 Timer1.Enabled := false;
  	   if(AutoViewDataTableCheckBox.Checked = true) then
  	   begin
  		 PageControl1.ActivePage := TabItemData;
  	   end; //if(AutoViewDataTableCheckBox.Checked = true)
       end //if(step_data_point >= number_of_measurements)
       else
       begin
  	 InstructionsLabel.Caption := 'Reaction times are: ' + LineEnding + '' + FloatToStr(stop_watch_visual_elapsed_in_ms) + ' ms (visual stimulus) ' + LineEnding + '' + FloatToStr(stop_watch_2nd_button_elapsed_in_ms) + ' ms (second button) ' + LineEnding + 'Press left mouse button for next measurement.';

  //     InstructionsLabel.Repaint();
  //     Application.ProcessMessages();


       end;//else if(step_data_point > number_of_measurements) then

    first_button_clicked := false; //MMMM
    second_button_clicked := true;

  end;//if(Button = TMouseButton.mbRight)


  end//if(Timer1.Enabled = true)



end;

procedure TMainForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin



  if(Timer1.Enabled = true) then
  begin
  // TMouseButton: mbLeft, mbRight, mbMiddle
  if(Button = TMouseButton.mbLeft) then
  begin



  if((stimulus_shown = false) and (first_button_clicked = true)) then
  begin

   first_button_clicked := false;

   stop_watch_2nd_button.Stop();
   stop_watch_2nd_button.Clear();

   stop_watch_delay_stimulus.Stop();
   stop_watch_delay_stimulus.Clear();

   {$IFDEF ANDROID}
   InstructionsLabel.Caption := 'Visual stimulus has not shown up yet ' + LineEnding + 'when you have released button. ' + LineEnding + 'Press left button to repeat.';
   {$ELSE}
   InstructionsLabel.Caption := 'Visual stimulus has not shown up yet ' + LineEnding + 'when you have released mouse button. ' + LineEnding + 'Press left mouse button to repeat.';
   {$ENDIF}


  end
  else //if((stimulus_shown = true) and (first_button_clicked = false) and (second_button_clicked = true)) then
  begin

   first_button_clicked := false;
   stimulus_shown := false;


   stop_watch_visual.Stop();

   stop_watch_2nd_button.Clear();
   stop_watch_2nd_button.Start();

   stop_watch_visual_elapsed_in_ms := RoundTo(stop_watch_visual.Elapsed * 1000.0, -3);

  end;//else if(stimulus_shown = false)


  CircleImage.Visible := false;


  InstructionsLabel.Repaint();
  Application.ProcessMessages();


  end;//if(Button = TMouseButton.mbLeft)

  end;//if(Timer1.Enabled = true)


end;

procedure TMainForm.FormResize(Sender: TObject);
begin


  GetScreenSize();


  FixationPointImage.Left := (main_form_width - FixationPointImage.Width) div 2;
  FixationPointImage.Top := (main_form_height - FixationPointImage.Height) div 2;


  CircleImage.Left := (main_form_width - CircleImage.Width) div 2;
  CircleImage.Top := (main_form_height - CircleImage.Height) div 2;


  LeftButtonRoundImage.Left := main_form_width div 2 - 30 - LeftButtonRoundImage.Width;
  RightButtonRoundImage.Left := main_form_width div 2 + 30;// RightButtonRoundImage.Width;

  LeftButtonRoundImage.Top := main_form_height - LeftButtonRoundImage.Height - 70;
  RightButtonRoundImage.Top := main_form_height - RightButtonRoundImage.Height - 70;

  left_button_rectangle_position_x := LeftButtonRoundImage.Left;
  left_button_rectangle_position_y := LeftButtonRoundImage.Top;

  right_button_rectangle_position_x := RightButtonRoundImage.Left;
  right_button_rectangle_position_y := RightButtonRoundImage.Top;


  left_button_rectangle_center_x := LeftButtonRoundImage.Left + LeftButtonRoundImage.Width div 2;
  left_button_rectangle_center_y := LeftButtonRoundImage.Top + LeftButtonRoundImage.Height div 2;

  right_button_rectangle_center_x := RightButtonRoundImage.Left + RightButtonRoundImage.Width div 2;
  right_button_rectangle_center_y := RightButtonRoundImage.Top + RightButtonRoundImage.Height div 2;



  //ClearDataTableButton.Left := 8;// main_form_width - ClearDataTableButton.Width;
  //ClearDataTableButton.Top := main_form_height - ClearDataTableButton.Height - 70;

  //CopyDataToClipboardButton.Left := main_form_width - DataStringGrid.Width - CopyDataToClipboardButton.Width - 40;
  //CopyDataToClipboardButton.Top := main_form_height - CopyDataToClipboardButton.Height - 70;

  //DataStringGrid.Left := main_form_width - DataStringGrid.Width - 30;
  DataStringGrid.Height := main_form_height - DataStringGrid.Top - 70;



  AboutMemo.Width := main_form_width - DataStringGrid.Left - 20;
  AboutMemo.Height := main_form_height - DataStringGrid.Top - 20;



  InstructionsLabel.Repaint();
  Application.ProcessMessages();

end;


procedure TMainForm.DelayForStimulusShowupStartEditEnter(Sender: TObject);
begin
  delay_for_stimulus_showup_start_saved := delay_for_stimulus_showup_start;
end;

procedure TMainForm.DelayForStimulusShowupStartEditExit(Sender: TObject);
begin


  delay_for_stimulus_showup_start := StrToIntDef(DelayForStimulusShowupStartEdit.Text, delay_for_stimulus_showup_start_default);

  if(delay_for_stimulus_showup_start >= delay_for_stimulus_showup_end) then
  begin
    ShowMessage('''from'' time must be less than ''to'' time');
    delay_for_stimulus_showup_start := delay_for_stimulus_showup_start_saved;
    DelayForStimulusShowupStartEdit.Text := IntToStr(delay_for_stimulus_showup_start);
    DelayForStimulusShowupStartEdit.SetFocus();
    DelayForStimulusShowupStartEdit.SelectAll();
  end
  else if((DelayForStimulusShowupStartEdit.Text = '') or (delay_for_stimulus_showup_start <= 0) or (delay_for_stimulus_showup_start > 100000)) then
  begin
    ShowMessage('Please enter value from 1 to 100000');
    delay_for_stimulus_showup_start := delay_for_stimulus_showup_start_saved;
    DelayForStimulusShowupStartEdit.Text := IntToStr(delay_for_stimulus_showup_start);
    DelayForStimulusShowupStartEdit.SetFocus();
    DelayForStimulusShowupStartEdit.SelectAll();
  end;


end;

procedure TMainForm.DelayForStimulusShowupStartEditKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  i: Integer;

begin

//https://lazarus-ccr.sourceforge.io/docs/lcl/lcltype/index-2.html
//Key codes

  if ((Key = VK_DELETE) or (Key = VK_BACK) or (Key = VK_LEFT) or (Key = VK_RIGHT)) then
  exit;


  if (Key = 13) then
  begin
    DelayForStimulusShowupStartEditExit(Self);
    exit;
  end;


// protection for length
  if (Length(DelayForStimulusShowupStartEdit.Text) > 61) then
  begin
     Key := 0;
     exit;
  end;

{
  if(DefaultFormatSettings.DecimalSeparator = '.') then
  begin
     if (Key = VK_LCL_COMMA) then
     begin
         Key := 0;
         exit;
     end
     else if (Key = VK_LCL_POINT) then
     begin
      for i := 1 to Length(DelayForStimulusShowupStartEdit.Text) do
      begin
        if(DelayForStimulusShowupStartEdit.Text[i] = '.') then
        begin
           Key := 0;
           exit;
        end;
      end;
      exit;
     end;
  end;


  if(DefaultFormatSettings.DecimalSeparator = ',') then
  begin
     if (Key = VK_LCL_POINT) then
     begin
         Key := 0;
         exit;
     end
     else if (Key = VK_LCL_COMMA) then
     begin
      for i := 1 to Length(DelayForStimulusShowupStartEdit.Text) do
      begin
       if(DelayForStimulusShowupStartEdit.Text[i] = ',') then
        begin
           Key := 0;
           exit;
        end;
      end;
      exit;
     end;
  end;
}

  if (((Key < VK_NUMPAD0) or (Key > VK_NUMPAD9)) and (((Key < VK_0) or (Key > VK_9))) ) then
  begin
    Key := 0;
    exit;
  end;


end;

procedure TMainForm.FixationPointImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseDown(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.FixationPointImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseUp(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin

    try
    ProgramIniFile := TIniFile.Create(ini_filename);

    if((number_of_measurements < 0) or (number_of_measurements > 10000)) then // protection from crash
    begin
    number_of_measurements := 30;
    end;

    ini_int_result := ProgramIniFile.ReadInteger( 'ReactionTime', 'number_of_measurements', 30);
    if(number_of_measurements <> ini_int_result) then
    begin

    ProgramIniFile.WriteInteger( 'ReactionTime', 'number_of_measurements', number_of_measurements);
    end;


    if((delay_for_stimulus_showup_start < 0) or (delay_for_stimulus_showup_start > 100000)) then // protection from crash
    begin
    delay_for_stimulus_showup_start := 1000;
    end;

    ini_int_result := ProgramIniFile.ReadInteger( 'ReactionTime', 'delay_for_stimulus_showup_start', 1000);
    if(delay_for_stimulus_showup_start <> ini_int_result) then
    begin

    ProgramIniFile.WriteInteger( 'ReactionTime', 'delay_for_stimulus_showup_start', delay_for_stimulus_showup_start);
    end;


    if((delay_for_stimulus_showup_end < 0) or (delay_for_stimulus_showup_end > 1000000)) then // protection from crash
    begin
    delay_for_stimulus_showup_end := 10000;
    end;


    ini_int_result := ProgramIniFile.ReadInteger( 'ReactionTime', 'delay_for_stimulus_showup_end', 10000);
    if((delay_for_stimulus_showup_end) <> ini_int_result) then
    begin

    ProgramIniFile.WriteInteger( 'ReactionTime', 'delay_for_stimulus_showup_end', delay_for_stimulus_showup_end);
    end;



    ini_bool_result := ProgramIniFile.ReadBool( 'ReactionTime', 'show_instructions', false);
    if(ShowInstructionsCheckBox.Checked <> ini_bool_result) then
    begin

    ProgramIniFile.WriteBool( 'ReactionTime', 'show_instructions', ShowInstructionsCheckBox.Checked);
    end;


    ini_bool_result := ProgramIniFile.ReadBool( 'ReactionTime', 'show_fixation_point', false);
    if(ShowFixationPointCheckBox.Checked <> ini_bool_result) then
    begin

    ProgramIniFile.WriteBool( 'ReactionTime', 'show_fixation_point', ShowFixationPointCheckBox.Checked);
    end;


    ini_bool_result := ProgramIniFile.ReadBool( 'ReactionTime', 'auto_view_data_table', false);
    if(AutoViewDataTableCheckBox.Checked <> ini_bool_result) then
    begin

    ProgramIniFile.WriteBool( 'ReactionTime', 'auto_view_data_table', AutoViewDataTableCheckBox.Checked);
    end;

    ini_bool_result := ProgramIniFile.ReadBool( 'ReactionTime', 'show_virtual_mouse_buttons', false);
    if(ShowVirtualMouseButtonsCheckBox.Checked <> ini_bool_result) then
    begin

    ProgramIniFile.WriteBool( 'ReactionTime', 'show_virtual_mouse_buttons', ShowVirtualMouseButtonsCheckBox.Checked);
    end;


    ini_string_result := ProgramIniFile.ReadString( 'ReactionTime', 'stimulus_color', '#FFFFFFFF');

    if(ColorToString(StimulurSLHColorPicker.SelectedColor) <> ini_string_result) then
    begin

    //ProgramIniFile.WriteString( 'ReactionTime', 'stimulus_color', ColorToString(StimulurSLHColorPicker.SelectedColor));
    ProgramIniFile.WriteInteger( 'ReactionTime', 'stimulus_color', StimulurSLHColorPicker.SelectedColor);
    end;


  finally
   ProgramIniFile.Free;
  end;


  CircleBitmap.Free;
  FixationPointBitmap.Free;
  VirtualMouseButtonRoundBitmap.Free;
  IncreasedVirtualMouseButtonRoundBitmap.Free;



end;


procedure TMainForm.DelayForStimulusShowupEndEditEnter(Sender: TObject);
begin
  delay_for_stimulus_showup_end_saved := delay_for_stimulus_showup_end;
end;

procedure TMainForm.DelayForStimulusShowupEndEditExit(Sender: TObject);
begin

  delay_for_stimulus_showup_end := StrToIntDef(DelayForStimulusShowupEndEdit.Text, delay_for_stimulus_showup_end_default);

  if(delay_for_stimulus_showup_start >= delay_for_stimulus_showup_end) then
  begin
    ShowMessage('''from'' time must be less than ''to'' time');
    delay_for_stimulus_showup_end := delay_for_stimulus_showup_end_saved;
    DelayForStimulusShowupEndEdit.Text := IntToStr(delay_for_stimulus_showup_end);
    DelayForStimulusShowupEndEdit.SetFocus();
    DelayForStimulusShowupEndEdit.SelectAll();
  end
  else if((DelayForStimulusShowupEndEdit.Text = '') or (delay_for_stimulus_showup_end <= 0) or (delay_for_stimulus_showup_end > 1000000)) then
  begin
    ShowMessage('Please enter value from 1 to 1000000');
    delay_for_stimulus_showup_end := delay_for_stimulus_showup_end_saved;
    DelayForStimulusShowupEndEdit.Text := IntToStr(delay_for_stimulus_showup_end);
    DelayForStimulusShowupEndEdit.SetFocus();
    DelayForStimulusShowupEndEdit.SelectAll();
  end;

end;

procedure TMainForm.DelayForStimulusShowupEndEditKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  i: Integer;

begin

//https://lazarus-ccr.sourceforge.io/docs/lcl/lcltype/index-2.html
//Key codes

  if ((Key = VK_DELETE) or (Key = VK_BACK) or (Key = VK_LEFT) or (Key = VK_RIGHT)) then
  exit;


  if (Key = 13) then
  begin
    DelayForStimulusShowupEndEditExit(Self);
    exit;
  end;


// protection for length
  if (Length(DelayForStimulusShowupEndEdit.Text) > 61) then
  begin
     Key := 0;
     exit;
  end;

{
  if(DefaultFormatSettings.DecimalSeparator = '.') then
  begin
     if (Key = VK_LCL_COMMA) then
     begin
         Key := 0;
         exit;
     end
     else if (Key = VK_LCL_POINT) then
     begin
      for i := 1 to Length(DelayForStimulusShowupEndEdit.Text) do
      begin
        if(DelayForStimulusShowupEndEdit.Text[i] = '.') then
        begin
           Key := 0;
           exit;
        end;
      end;
      exit;
     end;
  end;


  if(DefaultFormatSettings.DecimalSeparator = ',') then
  begin
     if (Key = VK_LCL_POINT) then
     begin
         Key := 0;
         exit;
     end
     else if (Key = VK_LCL_COMMA) then
     begin
      for i := 1 to Length(DelayForStimulusShowupEndEdit.Text) do
      begin
       if(DelayForStimulusShowupEndEdit.Text[i] = ',') then
        begin
           Key := 0;
           exit;
        end;
      end;
      exit;
     end;
  end;
}

  if (((Key < VK_NUMPAD0) or (Key > VK_NUMPAD9)) and (((Key < VK_0) or (Key > VK_9))) ) then
  begin
    Key := 0;
    exit;
  end;

end;


procedure TMainForm.ClearDataTableButtonClick(Sender: TObject);
var
 ai:Integer;
begin

  Timer1.Enabled := true;


  {$IFDEF ANDROID}
   InstructionsLabel.Caption := 'Press left button. ' + LineEnding + 'Keep left button pressed ' + LineEnding + 'until circle shows up. ' + LineEnding + 'Then release left button and press right ' + LineEnding + 'button as quickly as you can.';
  {$ELSE}
   InstructionsLabel.Caption := 'Press left mouse button. ' + LineEnding + 'Keep left mouse button pressed ' + LineEnding + 'until circle shows up. ' + LineEnding + 'Then release left mouse button and press right ' + LineEnding + 'mouse button as quickly as you can.';
  {$ENDIF}



  for ai := 0 to DataStringGrid.RowCount-1 do
  begin

   DataStringGrid.Cells[0, ai] := '';
   DataStringGrid.Cells[1, ai] := '';
   DataStringGrid.Cells[2, ai] := '';
  end;

  step_data_point := 1;

  PageControl1.ActivePage := TabItemExperiment;

end;

procedure TMainForm.CopyDataToClipboardButtonClick(Sender: TObject);
var
 temp_string2: UnicodeString;
 string_stream: TStringStream;
 i: Integer;
begin


  string_stream := TStringStream.Create(UnicodeString(''));

  for i := 1 to step_data_point - 1 do
  begin

  temp_string2 := DataStringGrid.Cells[0, i];
  string_stream.WriteString(temp_string2);

  string_stream.WriteString(char(9)); //horizontal TAB

  temp_string2 := DataStringGrid.Cells[1, i];
  string_stream.WriteString(temp_string2);

  string_stream.WriteString(char(9)); //horizontal TAB

  temp_string2 := DataStringGrid.Cells[2, i];
  string_stream.WriteString(temp_string2);
  string_stream.WriteString(' ' + LineEnding + '');

  end;//for (int i := 1; i < step_data_point; i++)

  string_stream.WriteString(char(0));
  string_stream.WriteString(char(0));
  string_stream.WriteString(char(0));


  Clipboard().AsText := string_stream.DataString;

  string_stream.Free;

end;

procedure TMainForm.BackgroundImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseUp(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.CircleImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin

   MainForm.OnMouseDown(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.CircleImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseUp(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.BackgroundImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseDown(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.BackgroundBlackPanelMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

   MainForm.OnMouseDown(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.BackgroundBlackPanelMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

   MainForm.OnMouseUp(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.FormShow(Sender: TObject);
begin

  Left := (Screen.Width - Width) div 2;
  Top := (Screen.Height - Height) div 2;




  {$IFDEF ANDROID}
  InstructionsLabel.Caption := 'Press left button.' + LineEnding + 'Keep left button pressed' + LineEnding + 'until circle shows up.' + LineEnding + 'Then release left button and press right' + LineEnding + 'button as quickly as you can.';
  {$ELSE}
  InstructionsLabel.Caption := 'Press left mouse button.' + LineEnding + 'Keep left mouse button pressed' + LineEnding + 'until circle shows up.' + LineEnding + 'Then release left mouse button and press right' + LineEnding + 'mouse button as quickly as you can.';
  {$ENDIF}


  WindowState := TWindowState.wsMaximized;

end;

procedure TMainForm.InstructionsLabelMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

     MainForm.OnMouseDown(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.InstructionsLabelMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseUp(Sender, Button, Shift, X, Y);

end;

procedure TMainForm.LeftButtonRoundImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

 MainForm.OnMouseDown(Sender, TMouseButton.mbLeft, Shift, X, Y);

 LeftButtonRoundImage.Width := increased_button_width;
 LeftButtonRoundImage.Height := increased_button_height;

 LeftButtonRoundImage.Left := left_button_rectangle_center_x - LeftButtonRoundImage.Width div 2;
 LeftButtonRoundImage.Top := left_button_rectangle_center_y - LeftButtonRoundImage.Height div 2;

 LeftButtonRoundImage.Picture.Bitmap.Assign(IncreasedVirtualMouseButtonRoundBitmap);

 //  left_button_rectangle_position_x := LeftButtonRoundImage.Left;
 //  left_button_rectangle_position_y := LeftButtonRoundImage.Top;

end;

procedure TMainForm.LeftButtonRoundImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

 MainForm.OnMouseUp(Sender, TMouseButton.mbLeft, Shift, X, Y);

 LeftButtonRoundImage.Width := button_width;
 LeftButtonRoundImage.Height := button_height;

 LeftButtonRoundImage.Left := left_button_rectangle_center_x - LeftButtonRoundImage.Width div 2;
 LeftButtonRoundImage.Top := left_button_rectangle_center_y - LeftButtonRoundImage.Height div 2;

 LeftButtonRoundImage.Picture.Bitmap.Assign(VirtualMouseButtonRoundBitmap);

 //  left_button_rectangle_position_x := LeftButtonRoundImage.Left;
 //  left_button_rectangle_position_y := LeftButtonRoundImage.Top;

end;


procedure TMainForm.NumberOfMeasurementsEditEnter(Sender: TObject);
begin
  number_of_measurements_saved := number_of_measurements;
end;

procedure TMainForm.NumberOfMeasurementsEditExit(Sender: TObject);
begin

  number_of_measurements := StrToIntDef(NumberOfMeasurementsEdit.Text, number_of_measurements_default);
  if((NumberOfMeasurementsEdit.Text = '') or  (number_of_measurements <= 0) or (number_of_measurements > 1000)) then
  begin
    ShowMessage('Please enter value from 1 to 1000');
    number_of_measurements := number_of_measurements_saved;
    NumberOfMeasurementsEdit.Text := IntToStr(number_of_measurements);
    NumberOfMeasurementsEdit.SetFocus();
    NumberOfMeasurementsEdit.SelectAll();
  end;

end;

procedure TMainForm.NumberOfMeasurementsEditKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
var
  i: Integer;

begin

//https://lazarus-ccr.sourceforge.io/docs/lcl/lcltype/index-2.html
//Key codes

  if ((Key = VK_DELETE) or (Key = VK_BACK) or (Key = VK_LEFT) or (Key = VK_RIGHT)) then
  exit;


  if (Key = 13) then
  begin
    NumberOfMeasurementsEditExit(Self);
    exit;
  end;


// protection for length
  if (Length(NumberOfMeasurementsEdit.Text) > 61) then
  begin
     Key := 0;
     exit;
  end;

 {
  if(DefaultFormatSettings.DecimalSeparator = '.') then
  begin
     if (Key = VK_LCL_COMMA) then
     begin
         Key := 0;
         exit;
     end
     else if (Key = VK_LCL_POINT) then
     begin
      for i := 1 to Length(NumberOfMeasurementsEdit.Text) do
      begin
        if(NumberOfMeasurementsEdit.Text[i] = '.') then
        begin
           Key := 0;
           exit;
        end;
      end;
      exit;
     end;
  end;


  if(DefaultFormatSettings.DecimalSeparator = ',') then
  begin
     if (Key = VK_LCL_POINT) then
     begin
         Key := 0;
         exit;
     end
     else if (Key = VK_LCL_COMMA) then
     begin
      for i := 1 to Length(NumberOfMeasurementsEdit.Text) do
      begin
       if(NumberOfMeasurementsEdit.Text[i] = ',') then
        begin
           Key := 0;
           exit;
        end;
      end;
      exit;
     end;
  end;
 }

  if (((Key < VK_NUMPAD0) or (Key > VK_NUMPAD9)) and (((Key < VK_0) or (Key > VK_9))) ) then
  begin
    Key := 0;
    exit;
  end;

end;

procedure TMainForm.RightButtonRoundImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseDown(Sender, TMouseButton.mbRight, Shift, X, Y);

  RightButtonRoundImage.Width := increased_button_width;
  RightButtonRoundImage.Height := increased_button_height;

  RightButtonRoundImage.Left := right_button_rectangle_center_x - RightButtonRoundImage.Width div 2;
  RightButtonRoundImage.Top := right_button_rectangle_center_y - RightButtonRoundImage.Height div 2;

  RightButtonRoundImage.Picture.Bitmap.Assign(IncreasedVirtualMouseButtonRoundBitmap);

  //  right_button_rectangle_position_x := RightButtonRoundImage.Left;
  //  right_button_rectangle_position_y := RightButtonRoundImage.Top;

end;

procedure TMainForm.RightButtonRoundImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin

  MainForm.OnMouseUp(Sender, TMouseButton.mbRight, Shift, X, Y);

  RightButtonRoundImage.Width := button_width;
  RightButtonRoundImage.Height := button_height;

  RightButtonRoundImage.Left := right_button_rectangle_center_x - RightButtonRoundImage.Width div 2;
  RightButtonRoundImage.Top := right_button_rectangle_center_y - RightButtonRoundImage.Height div 2;

  RightButtonRoundImage.Picture.Bitmap.Assign(VirtualMouseButtonRoundBitmap);

  //  right_button_rectangle_position_x := RightButtonRoundImage.Left;
  //  right_button_rectangle_position_y := RightButtonRoundImage.Top;

end;

procedure TMainForm.ShowFixationPointCheckBoxChange(Sender: TObject);
begin

  FixationPointImage.Visible := ShowFixationPointCheckBox.Checked;

end;

procedure TMainForm.ShowInstructionsCheckBoxChange(Sender: TObject);
begin

  InstructionsLabel.Visible := ShowInstructionsCheckBox.Checked;

end;

procedure TMainForm.ShowVirtualMouseButtonsCheckBoxChange(Sender: TObject);
begin


  show_virtual_mouse_buttons := ShowVirtualMouseButtonsCheckBox.Checked;
  RightButtonRoundImage.Visible := show_virtual_mouse_buttons;
  LeftButtonRoundImage.Visible := show_virtual_mouse_buttons;

end;

procedure TMainForm.StimulurSLHColorPickerChange(Sender: TObject);
begin


  CircleBitmap.CanvasBGRA.Pen.Color := StimulurSLHColorPicker.SelectedColor;
  CircleBitmap.CanvasBGRA.Pen.Style := psSolid;
  CircleBitmap.CanvasBGRA.Pen.Width := 0;
  CircleBitmap.CanvasBGRA.Brush.Color := StimulurSLHColorPicker.SelectedColor;
  CircleBitmap.CanvasBGRA.Brush.Style := bsSolid;
  CircleBitmap.CanvasBGRA.Ellipse(0, 0, CircleBitmap.Width, CircleBitmap.Height);

//  CircleImage.Width := CircleBitmap.Width;
//  CircleImage.Height := CircleBitmap.Height;
  CircleImage.Picture.Bitmap.Assign(CircleBitmap);


end;

procedure TMainForm.TabItemOptionsShow(Sender: TObject);
begin

  //ShowMessage('TabItemOptionsShow');
  DelayForStimulusShowupStartEdit.OnExit := nil;
  DelayForStimulusShowupEndEdit.OnExit := nil;
  NumberOfMeasurementsEdit.OnExit := nil;

  ShowInstructionsCheckBox.Checked := InstructionsLabel.Visible;
  ShowFixationPointCheckBox.Checked := FixationPointImage.Visible;

  number_of_measurements_saved := number_of_measurements;
  delay_for_stimulus_showup_start_saved := delay_for_stimulus_showup_start;
  delay_for_stimulus_showup_end_saved := delay_for_stimulus_showup_end;

  NumberOfMeasurementsEdit.Text := IntToStr(number_of_measurements);
  DelayForStimulusShowupStartEdit.Text := IntToStr(delay_for_stimulus_showup_start);
  DelayForStimulusShowupEndEdit.Text := IntToStr(delay_for_stimulus_showup_end);

  DelayForStimulusShowupStartEdit.OnExit := @DelayForStimulusShowupStartEditExit;
  DelayForStimulusShowupEndEdit.OnExit := @DelayForStimulusShowupEndEditExit;
  NumberOfMeasurementsEdit.OnExit := @NumberOfMeasurementsEditExit;

end;


procedure TMainForm.Timer1Timer(Sender: TObject);
begin



  if(first_button_clicked = true) then
  begin


  if(stimulus_shown = false) then
  begin



  stop_watch_delay_stimulus.Stop();

  if (delay_for_stimulus_showup_in_seconds < stop_watch_delay_stimulus.Elapsed) then
  begin

  delay_for_stimulus_showup := 0;
  stimulus_shown := true;

  stop_watch_delay_stimulus.Clear();

  CircleImage.Visible := true;

  stop_watch_visual.Clear();
  stop_watch_visual.Start();


  end;//if (delay_for_stimulus_showup_in_seconds < stop_watch_delay_stimulus.Elapsed)


  stop_watch_delay_stimulus.Start();

  end//if(stimulus_shown = false)


  end//if(first_button_clicked = true)

end;

end.

